package com.hiyuyi.library.plugin;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hiyuyi.library.base.log.YyLog;
import com.hiyuyi.library.base.external.BaseExternal;
import com.hiyuyi.library.base.external.LibGlobal;
import com.hiyuyi.library.base.external.Request;
import com.hiyuyi.library.base.external.Response;
import com.hiyuyi.library.base.iml.Callback;
import com.hiyuyi.library.base.iml.Callback2;
import com.hiyuyi.library.base.net.api.Api;
import com.hiyuyi.library.base.net.api.ApiModel;
import com.hiyuyi.library.base.net.api.ApiRespons;
import com.hiyuyi.library.base.net.api.ApiType;
import com.hiyuyi.library.base.singleton.Singleton;
import com.hiyuyi.library.base.thread.ThreadPool;
import com.hiyuyi.library.base.utils.BaseUtils;
import com.hiyuyi.library.base.utils.EncryptUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dalvik.system.DexClassLoader;

/**
 * @author JuanTop
 * @date 2021/6/27
 */
public class DexLoader {

    public static final int STATUS_NEED_DOWNLOAD = 10;
    public static final int STATUS_FAIL = 11;
    public static final int STATUS_SUCCESS = 12;
    public static final int STATUS_COMPLETE = 13;


    private final DexMerge dexMerge;
    private final DexLocal dexLocal;

    private static final Object lockObject;
    private static boolean checkUpdateIsFinish;

    static {
        lockObject = new Object();
        checkUpdateIsFinish = false;
    }

    private final Handler mHandler = new Handler(Looper.getMainLooper());


    private DexLoader() {
        dexMerge = new DexMerge();
        dexLocal = new DexLocal();
    }

    private static final Singleton<DexLoader> ISingleton = new Singleton<DexLoader>() {
        @Override
        protected DexLoader create() {
            return new DexLoader();
        }
    };

    public static DexLoader get() {
        return ISingleton.get();
    }

    /**
     * 检测是否有插件更新
     *
     * @param callback
     */
    public void checkUpdate(Callback2<Integer, String> callback) {
        synchronized (lockObject) {
            checkUpdateIsFinish = true;
        }
        // 高版本未适配，关闭此功能
        callback.callback(STATUS_COMPLETE, "");
//        ThreadPool.get().execute(() -> {
//            try {
//                Map<String, Integer> map = BaseExternal.pckAndVersion;
//                DexRequest dexRequest = new DexRequest();
//                for (String key : map.keySet()) {
//                    //获取本地的lib 插件版本跟代码里的版本，取大的那个，上传给服务端
//                    int localVersion = dexLocal.getLocalVersion(key);
//                    Integer codeVersionInt = map.get(key);
//                    int codeVersion = codeVersionInt == null ? 0 : codeVersionInt;
//                    int version = Math.max(localVersion, codeVersion);
//                    //获取本地的插件 md5
//                    String localMd5 = dexLocal.getLocalMd5(key);
//
//                    // 封装请求参数
//                    dexRequest.getLiData().add(DexRequest.LiDataBean.newInstance(key, version, localMd5));
//                }
//
//                List<DexResponse> dexResponses = getDexResult(dexRequest);
//                if (dexResponses == null || dexResponses.isEmpty()) {
//                    callback(aBoolean -> callback.callback(STATUS_COMPLETE, "接口数据显示没有可更新的插件"));
//                    return;
//                }
//                List<DexResponse> downloadResponses = new ArrayList<>();
//                List<DexResponse> mergeResponses = new ArrayList<>();
//                for (DexResponse dexResponse : dexResponses) {
//                    if (dexResponse.needUpgrade()) {
//                        downloadResponses.add(dexResponse);
//                    }
//                    if (dexResponse.canMigrate()) {
//                        mergeResponses.add(dexResponse);
//                    }
//                }
//                //下载插件
//                if (downloadResponses.isEmpty()) {
//                    YyLog.e("plugin not need update");
//                } else {
//                    YyLog.e("plugin need update , plugin size:" + downloadResponses.size());
//                    callback(aBoolean -> callback.callback(STATUS_NEED_DOWNLOAD, ""));
//                    boolean downloadResult = downloadPlugin(downloadResponses);
//                    if (!downloadResult) {
//                        callback(aBoolean -> callback.callback(STATUS_FAIL, "下载插件失败"));
//                    }
//                }
//                //合并插件
//                if (mergeResponses.isEmpty()) {
//                    YyLog.e("plugin not need merge");
//                } else {
//                    YyLog.e("plugin need merge , plugin size:" + mergeResponses.size());
//                    boolean mergeResult = mergeDex(mergeResponses);
//                    if (!mergeResult) {
//                        YyLog.e("合并失败");
//                        callback(aBoolean -> callback.callback(STATUS_FAIL, "合并失败"));
//                    }
//                }
//
//                callback(aBoolean -> {
//                    callback.callback(STATUS_SUCCESS, "插件下发合并成功");
//                    callback.callback(STATUS_COMPLETE, "");
//                });
//            } catch (Exception e) {
//                callback(aBoolean -> callback.callback(STATUS_COMPLETE, ""));
//            }
//        });
    }

    public List<DexResponse> getDexResult(DexRequest dexRequest) throws Exception {
        //判断 app 版本号，如果是老版本需要清楚本地的所有插件，重新下载
        String appVersion = getAppVersionName();
        if (!TextUtils.equals(appVersion, BaseUtils.getVersionName())) {
            saveResponse("");
            dexLocal.clearAllDexFile(dexRequest);
        }
        //请求网络，判断是否需要更新
        ApiRespons apiRespons = Api.api(ApiModel.value("wsv3", "api/v1/arr-file/fetch-arr-info", ApiType.PLUGIN, JSON.toJSONString(dexRequest))).sync();
        String responseJson;
        if (apiRespons.code == ApiRespons.CODE_FAILE) {
            YyLog.e("plugin inter false, please check net");
            responseJson = getResponse();
        } else {
            responseJson = apiRespons.data;
            saveResponse(responseJson);
            saveAppVersionName();
        }
        //非对称解密
        JSONObject jsonObject = JSONObject.parseObject(responseJson);
//        String dataJsonObjectStr = jsonObject.getString("data");
//        JSONObject dataJsonObject = JSONObject.parseObject(dataJsonObjectStr).getJSONObject("data");
        String data = jsonObject.getString("data");
        String key = jsonObject.getString("key");
        String password = EncryptUtils.decryptRSA(key);
        YyLog.e("password:" + password);
        String result = EncryptUtils.decryptAES(data, password);
        YyLog.e("解密之后的服务端数据->result:" + result);
        return JSON.parseArray(result, DexResponse.class);
    }

    public void saveResponse(String json) {
        Bundle bundle = new Bundle();
        bundle.putString("fileName", "dex_file");
        bundle.putString("key", "dex_key");
        bundle.putString("value", json);
        BaseExternal.syncObtain(Request.newInstance(LibGlobal.LIB_STORAGE, LibGlobal.MethodStorageGlobal.SAVE_TO_SP, bundle));
    }

    public String getResponse() {
        Bundle bundle = new Bundle();
        bundle.putString("fileName", "dex_file");
        bundle.putString("key", "dex_key");
        bundle.putString("defaultValue", "");
        Response response = BaseExternal.syncObtain(Request.newInstance(LibGlobal.LIB_STORAGE, LibGlobal.MethodStorageGlobal.GET_TO_SP, bundle));
        if (response == null || response.data == null) {
            return "";
        }
        return (String) response.data;
    }

    public void saveAppVersionName() {
        Bundle bundle = new Bundle();
        bundle.putString("fileName", "dex_file");
        bundle.putString("key", "dex_app_version_key");
        bundle.putString("value", BaseUtils.getVersionName());
        BaseExternal.syncObtain(Request.newInstance(LibGlobal.LIB_STORAGE, LibGlobal.MethodStorageGlobal.SAVE_TO_SP, bundle));
    }

    public String getAppVersionName() {
        Bundle bundle = new Bundle();
        bundle.putString("fileName", "dex_file");
        bundle.putString("key", "dex_app_version_key");
        bundle.putString("defaultValue", "");
        Response response = BaseExternal.syncObtain(Request.newInstance(LibGlobal.LIB_STORAGE, LibGlobal.MethodStorageGlobal.GET_TO_SP, bundle));
        if (response == null || response.data == null) {
            return "";
        }
        return (String) response.data;
    }

    /**
     * 一个一个下载
     *
     * @param dexResponses
     * @return
     */
    public boolean downloadPlugin(List<DexResponse> dexResponses) {
        Map<String, String> map = new HashMap<>();
        boolean downloadResult = true;
        for (DexResponse dexResponse : dexResponses) {
            String libPck = dexResponse.getFunctionId();
            if (TextUtils.isEmpty(libPck)) {
                continue;
            }
            dexLocal.clearTempFile(libPck);

            String tempFilePath = dexLocal.getTempDexFilePath(libPck, String.valueOf(dexResponse.getInnerVersion()));
            map.clear();
            map.put(dexResponse.getFileUrl(), tempFilePath);
            Request request = Request.newInstance(LibGlobal.LIB_NETWORK, LibGlobal.MethodNetworkGlobal.SYNC_DOWNLOAD, map);
            Response response = BaseExternal.syncObtain(request);
            if (response == null) {
                YyLog.e("downloadPlugin fail");
                downloadResult = false;
            }
            //校验下载文件的完整性
            boolean checkResult = dexLocal.checkFile(tempFilePath, libPck, String.valueOf(dexResponse.getInnerVersion()), dexResponse.getServerMd5());
            if (!checkResult) {
                YyLog.e("下载文件不完整，libPck:" + libPck);
                downloadResult = false;
                dexLocal.clearTempFile(libPck);
                continue;
            }
            //解密
            File tempFile = new File(tempFilePath);
            dexLocal.clearDexFile(libPck);
            String dexFilePath = dexLocal.getDexFilePath(libPck, String.valueOf(dexResponse.getInnerVersion()));
            try {
                byte[] tempByte = EncryptUtils.readFile(tempFile);
                File dexFile = new File(dexFilePath);
                byte[] dexByte = EncryptUtils.decryptAES(tempByte, dexResponse.getKey());

                EncryptUtils.writeFile(dexFile, dexByte);
            } catch (Exception e) {
                YyLog.e("解密文件异常：" + e.toString());
                downloadResult = false;
                dexLocal.clearDexFile(libPck);
                dexLocal.clearTempFile(libPck);
                continue;
            }
            dexLocal.clearTempFile(libPck);
            //最后再验证一次文件的 md5
            boolean checkResult2 = dexLocal.checkFile(dexFilePath, libPck, String.valueOf(dexResponse.getInnerVersion()), dexResponse.getLocalMd5());
            if (!checkResult2) {
                YyLog.e("解密之后，文件md5不对，libPck2:" + libPck);
                downloadResult = false;
                dexLocal.clearDexFile(libPck);
                continue;
            }
            YyLog.e("插件 " + libPck + " 下载校验都已经通过");
        }
        return downloadResult;
    }

    public boolean mergeDex(List<DexResponse> dexResponses) {
        boolean mergeResult = true;
        List<DexClassLoader> dexClassLoaders = new ArrayList<>();
        for (DexResponse dexResponse : dexResponses) {
            String dexFilePath = dexLocal.getDexFilePath(dexResponse.getFunctionId(), String.valueOf(dexResponse.getInnerVersion()));
            if (TextUtils.isEmpty(dexFilePath)) {
                continue;
            }
            File dexFile = new File(dexFilePath);
            if (!dexFile.exists()) {
                continue;
            }
            String dexFileParentPath = dexFile.getParent();
            DexClassLoader dexClassLoader = dexMerge.getDexClassLoader(dexFilePath, dexFileParentPath, YyInter.application.getClassLoader());
            dexClassLoaders.add(dexClassLoader);
        }
        DexClassLoader[] dexClassLoader = new DexClassLoader[dexClassLoaders.size()];
        dexClassLoaders.toArray(dexClassLoader);
        boolean temp = dexMerge.mergeDex(YyInter.application, dexClassLoader);
        if (!temp) {
            mergeResult = false;
        }
        return mergeResult;
    }

    private void callback(Callback<Boolean> callback) {
        mHandler.post(() -> callback.callback(true));
    }

    public static boolean check() {
        boolean checkResult;
        synchronized (lockObject) {
            checkResult = checkUpdateIsFinish;
            return checkResult;
        }
    }
}
