package com.hiyuyi.library.plugin;

import android.text.TextUtils;

import com.hiyuyi.library.base.log.YyLog;
import com.hiyuyi.library.base.utils.EncryptUtils;
import com.hiyuyi.library.base.utils.FileUtils;

import java.io.File;
import java.io.IOException;

/**
 * @author JuanTop
 * @date 2021/6/27
 */
public class DexLocal {
    private final String dexFileParentPath;
    private final String tempDexFileParentPath;

    public DexLocal() {
        String rootPath = YyInter.application.getFilesDir().getAbsolutePath();
        File dexFileParent = new File(rootPath, "dex");
        dexFileParentPath = dexFileParent.getAbsolutePath();

        if (!dexFileParent.exists()) {
            boolean createDexDir = dexFileParent.mkdir();
            YyLog.e("DexLocal[DexLocal]: createDexDir:" + createDexDir);
        }
        File tempDexFileParent = new File(rootPath, "temp_dex");
        tempDexFileParentPath = tempDexFileParent.getAbsolutePath();
        if (!tempDexFileParent.exists()) {
            boolean createTempDeDir = tempDexFileParent.mkdir();
            YyLog.e("DexLocal[DexLocal]: createTempDeDir:" + createTempDeDir);
        }
    }

    public int getLocalVersion(String libPck) {
        File targetFile = getDirByLibPck(dexFileParentPath, libPck);
        if (targetFile == null) {
            return 0;
        }
        File file = getJarFileForMaxVersion(targetFile.getAbsolutePath());
        if (file == null) {
            return 0;
        }
        return getJarFileVersion(file);
    }

    public String getLocalMd5(String libPck) {
        File targetFile = getDirByLibPck(dexFileParentPath, libPck);
        if (targetFile == null) {
            return "";
        }
        File file = getJarFileForMaxVersion(targetFile.getAbsolutePath());
        if (file == null) {
            return "";
        }
        return EncryptUtils.getFileMd5(file);
    }

    public String getDexFilePath(String libPck, String defaultFileName) {
        File targetFile = getDirByLibPck(dexFileParentPath, libPck);
        if (targetFile == null) {
            targetFile = new File(dexFileParentPath + "/" + libPck);
            targetFile.mkdir();
        }
        File file = getJarFileForMaxVersion(targetFile.getAbsolutePath());
        if (file == null) {
            file = new File(targetFile.getAbsoluteFile() + "/" + defaultFileName + ".jar");
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return file.getAbsolutePath();
    }

    public String getTempDexFilePath(String libPck, String defaultFileName) {
        File targetFile = getDirByLibPck(tempDexFileParentPath, libPck);
        if (targetFile == null) {
            targetFile = new File(tempDexFileParentPath + "/" + libPck);
            boolean createFileDir = targetFile.mkdirs();
            YyLog.e("DexLocal[getTempDexFilePath]createFileDir: " + createFileDir);
        }
        File file = getJarFileForMaxVersion(targetFile.getAbsolutePath());
        if (file == null) {
            file = new File(targetFile.getAbsoluteFile() + "/" + defaultFileName + ".jar");
            try {
                boolean createFile = file.createNewFile();
                YyLog.e("DexLocal[getTempDexFilePath]createFile: " + createFile);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return file.getAbsolutePath();
    }

    public boolean checkFile(String filePath, String libPck, String version, String md5) {
        if (TextUtils.isEmpty(filePath)) {
            return false;
        }
        File file = new File(filePath);
        if (!file.exists()) {
            return false;
        }
        String fileAbsolutePath = file.getAbsolutePath();
        if (!fileAbsolutePath.contains(libPck) || !fileAbsolutePath.contains(version)) {
            return false;
        }
        if (!TextUtils.equals(EncryptUtils.getFileMd5(file), md5)) {
            return false;
        }
        return true;
    }

    public void clearTempFile(String libPck) {
        File targetFile = getDirByLibPck(tempDexFileParentPath, libPck);
        if (targetFile == null) {
            return;
        }
        File[] files = targetFile.listFiles();
        if (files == null || files.length == 0) {
            return;
        }
        for (File file : files) {
            file.delete();
        }
    }

    public void clearDexFile(String libPck) {
        File targetFile = getDirByLibPck(dexFileParentPath, libPck);
        if (targetFile == null) {
            return;
        }
        File[] files = targetFile.listFiles();
        if (files == null || files.length == 0) {
            return;
        }
        for (File file : files) {
            file.delete();
        }
    }

    public void clearAllDexFile(DexRequest dexRequest) {
        for (DexRequest.LiDataBean liDatum : dexRequest.getLiData()) {
            clearTempFile(liDatum.getFunctionId());
            clearDexFile(liDatum.getFunctionId());
        }
//        FileUtils.deleteDirWithFile(new File(tempDexFileParentPath));
//        FileUtils.deleteDirWithFile(new File(dexFileParentPath));
    }

    private File getDirByLibPck(String parentPath, String libPck) {
        File parentFile = new File(parentPath);
        if (!parentFile.exists()) {
            return null;
        }
        File[] files = parentFile.listFiles();
        if (files == null || files.length == 0) {
            return null;
        }
        for (File file : files) {
            if (!file.isDirectory()) {
                continue;
            }
            if (TextUtils.equals(file.getName(), libPck)) {
                return file;
            }
        }
        return null;
    }

    private File getJarFileForMaxVersion(String parentPath) {
        File parentFile = new File(parentPath);
        if (!parentFile.exists()) {
            return null;
        }
        File[] files = parentFile.listFiles();
        if (files == null || files.length == 0) {
            return null;
        }
        int version = 0;
        File targetFile = null;
        for (File file : files) {
            if (!file.isFile()) {
                continue;
            }
            String name = file.getName();
            if (!name.endsWith(".jar")) {
                continue;
            }
            int temp = getJarFileVersion(file);
            if (version < temp) {
                targetFile = file;
                version = temp;
            }
        }
        return targetFile;
    }

    private int getJarFileVersion(File file) {
        try {
            String name = file.getName();
            return Integer.parseInt(name.substring(0, name.lastIndexOf(".")));
        } catch (Exception ignored) {
        }
        return 0;
    }

}
