package com.hiyuyi.library.plugin;

import android.content.Context;

import com.hiyuyi.library.base.log.YyLog;

import java.lang.reflect.Array;
import java.lang.reflect.Field;

import dalvik.system.DexClassLoader;

/**
 * @author JuanTop
 * @date 2021/6/27
 */
public class DexMerge {

    public boolean mergeDex(Context context, DexClassLoader... dexClassLoader) {
        try {
            Object[] fixDexElements = new Object[dexClassLoader.length];
            for (int i = 0; i < dexClassLoader.length; i++) {
                Object dexObj = getPathList(dexClassLoader[i]);//load-jar
                fixDexElements[i] = getDexElements(dexObj);
            }
            Object pathObj = getPathList(context.getClassLoader());//jar
            Object pathDexElements = getDexElements(pathObj);
            Object newDexElements = combineArray(pathDexElements, fixDexElements);
            //重新赋值给PathClassLoader 中的exElements数组
            Object pathList = getPathList(context.getClassLoader());
            setField(pathList, pathList.getClass(), "dexElements", newDexElements);
            return true;
        } catch (Exception e) {
            YyLog.e("mergeDex exception:" + e.toString());
            e.printStackTrace();
            return false;
        }
    }

    public DexClassLoader getDexClassLoader(String dexFilePath, String dexParentPath, ClassLoader classLoader) {
        return new DexClassLoader(dexFilePath, dexParentPath, null, classLoader);
    }

    private Object getPathList(Object baseDexClassLoader) throws Exception {
        return getField(baseDexClassLoader, Class.forName("dalvik.system.BaseDexClassLoader"), "pathList");
    }

    private Object getDexElements(Object obj) throws Exception {
        return getField(obj, obj.getClass(), "dexElements");
    }

    private Object getField(Object obj, Class<?> cl, String field) throws NoSuchFieldException, IllegalAccessException {
        Field localField = cl.getDeclaredField(field);
        localField.setAccessible(true);
        return localField.get(obj);
    }

    private void setField(Object obj, Class<?> cl, String field, Object value) throws Exception {
        Field localField = cl.getDeclaredField(field);
        localField.setAccessible(true);
        localField.set(obj, value);
    }

    private Object combineArray(Object arrayLhs, Object... arrayRhs) {
        Class<?> localClass = arrayLhs.getClass().getComponentType();
        int i = Array.getLength(arrayLhs);
        int j = i;
        for (Object arrayRh : arrayRhs) {
            j += Array.getLength(arrayRh);
        }
        Object result = Array.newInstance(localClass, j);
        int index = 0;
        for (Object arrayRh : arrayRhs) {
            for (int i1 = 0; i1 < Array.getLength(arrayRh); i1++) {
                Array.set(result, index, Array.get(arrayRh, i1));
                index++;
            }
        }
        for (int i1 = 0; i1 < i; i1++) {
            Array.set(result, index, Array.get(arrayLhs, i1));
            index++;
        }
        return result;
    }
}
